package sdk_996.bean.cnofig;

import com.alibaba.fastjson.JSONObject;
import org.postgresql.util.PSQLException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import org.springframework.web.bind.annotation.ControllerAdvice;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.ResponseBody;
import sdk_996.bean.em.ErrorCode;
import sdk_996.bean.exp.FormatValidationException;
import sdk_996.bean.exp.SqlException;
import sdk_996.bean.vo.ResultVO;

import java.net.SocketTimeoutException;
import java.util.concurrent.CompletionException;

@ControllerAdvice
public class GlobalExceptionHandler {

    private static final Logger logger = LoggerFactory.getLogger(GlobalExceptionHandler.class);


    @ExceptionHandler(value = {FormatValidationException.class})
    @ResponseBody
    public ResultVO formatValidationException(FormatValidationException ex) {
        logger.info("format validation exception: {}", ex);
        return ResultVO.error(ErrorCode.PARAM_ERROR);
    }


    @ExceptionHandler(value = {SqlException.class, CompletionException.class})
    @ResponseBody
    public ResultVO handleSqlException(SqlException ex) {
        Exception cause = ex.getEx();
        String key_v = ex.getKey_v();

        // 根据具体的异常类型做出响应
        if (cause.getCause().getCause() instanceof SocketTimeoutException) {
            return handleTimeoutException(key_v);
        } else if (cause.getCause() instanceof PSQLException) {
            return handleSqlSyntaxException(key_v, cause);
        } else {
            return handleGenericSqlException(key_v, cause);
        }
    }

    private ResultVO handleTimeoutException(String key_v) {
        logger.error("SQL query timeout for key: {}", key_v);
        JSONObject errorDetail = createErrorDetail(key_v, ErrorCode.SQL_TIMEOUT.getMessage(), "");
        return ResultVO.error_sql(ErrorCode.SQL_TIMEOUT, errorDetail);
    }

    private ResultVO handleSqlSyntaxException(String sqlKey, Exception cause) {
        logger.error("SQL syntax error for key: {}", sqlKey, cause);
        JSONObject errorDetail = createErrorDetail(sqlKey, ErrorCode.SQL_ERROR.getMessage(), cause.getMessage());
        return ResultVO.error_sql(ErrorCode.SQL_ERROR, errorDetail);
    }

    private ResultVO handleGenericSqlException(String sqlKey, Exception cause) {
        logger.error("SQL execution error for key: {}", sqlKey, cause);
        return ResultVO.error(ErrorCode.SQL_EXECUTE_ERROR);
    }

    private JSONObject createErrorDetail(String sqlKey, String Message, String errorMessage) {
        JSONObject jsonObject = new JSONObject();
        jsonObject.put("sql_key:sql", sqlKey);
        jsonObject.put(Message, errorMessage);
        return jsonObject;
    }

    @ExceptionHandler(value = ArithmeticException.class)
    @ResponseBody
    public ResultVO handleArithmeticException(ArithmeticException ex) {
        logger.error("Arithmetic exception occurred: ", ex);
        return ResultVO.error(ErrorCode.CODE_ERROR);
    }

    @ExceptionHandler(value = NullPointerException.class)
    @ResponseBody
    public ResultVO handleNullPointerException(NullPointerException ex) {
        logger.error("NullPointerException occurred: ", ex);
        return ResultVO.error(ErrorCode.NULL_POINTER_ERROR);
    }

    @ExceptionHandler(value = Exception.class)
    @ResponseBody
    public ResultVO handleGeneralException(Exception ex) {
        logger.error("General exception occurred: ", ex);
        return ResultVO.error(ErrorCode.RUNTIME_EXCEPTION);
    }
}
