package sdk_996.bean.vo;

import com.alibaba.fastjson.JSONObject;
import lombok.Data;
import sdk_996.bean.em.ErrorCode;

import java.time.LocalDateTime;

@Data
public class ResultVO {
    private Integer code;
    private Object message;
    private Object data;
    private LocalDateTime currentTime; // 当前时间字段

    public ResultVO() {
        this.currentTime = LocalDateTime.now(); // 在构造函数中设置当前时间
        this.setCode(200); // 默认成功状态
        this.setMessage("成功"); // 默认成功消息
    }

    // 提供一个静态方法来快速创建成功响应
    public static ResultVO success() {
        return new ResultVO();
    }

    // 提供一个静态方法来快速创建带有自定义消息的成功响应
    public static ResultVO success(JSONObject data) {
        ResultVO resultVO = new ResultVO();
        resultVO.setData(data);
        return resultVO;
    }

    // 提供一个静态方法来快速创建带有错误码和消息的响应
    public static ResultVO error(ErrorCode errorCode) {
        ResultVO resultVO = new ResultVO();
        resultVO.setCode(errorCode.getCode());
        resultVO.setMessage(errorCode.getMessage());
        return resultVO;
    }


    public static ResultVO error_sql(ErrorCode errorCode, JSONObject msg) {
        ResultVO resultVO = new ResultVO();
        resultVO.setCode(errorCode.getCode());
        resultVO.setMessage(msg);
        return resultVO;
    }

    public static void main(String[] args) {
        ResultVO resultVO = ResultVO.error(ErrorCode.SQL_ERROR);
        System.out.println(resultVO);
    }

    @Override
    public String toString() {
        // 以json格式打印返回
        return JSONObject.toJSONString(this);
    }
}
