package sdk_996.service;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import sdk_996.bean.exp.FormatValidationException;
import sdk_996.mapper.SqlExecuteMapper;
import sdk_996.bean.exp.SqlException;
import com.alibaba.fastjson.JSON;

import java.util.List;
import java.util.Map;
import java.util.concurrent.CompletableFuture;
import java.util.stream.Collectors;

@Service
public class SqlExecuteService {


    private static final Logger logger = LoggerFactory.getLogger(SqlExecuteService.class);

    @Autowired
    private SqlExecuteMapper sqlExecuteMapper;

    public JSONObject performCustomQuery(String sqlstr) {
        // 返回的json对象
        JSONObject returnJsonObject = new JSONObject();
        JSONObject sqlJsonObject;

        try {
            // 将SQL字符串转换为JSON对象
            sqlJsonObject = JSONObject.parseObject(sqlstr);

        } catch (Exception ex) {
            throw new FormatValidationException(ex);
        }


        // 执行查询操作并处理返回结果
        List<JSONObject> jsonObjects = query(sqlJsonObject);

        // 遍历jsonObjects，将结果加入到返回的jsonObject
        for (JSONObject jsonObject : jsonObjects) {
            jsonObject.forEach(returnJsonObject::put);
        }

        return returnJsonObject;


    }

    private List<JSONObject> query(JSONObject sqlJsonObject) {
        // 执行所有 SQL 查询的异步任务
        List<CompletableFuture<JSONObject>> futures = sqlJsonObject.keySet().stream()
                .map(key -> CompletableFuture.supplyAsync(() -> executeQuery(key, sqlJsonObject.getString(key))))
                .collect(Collectors.toList());

        // 等待所有任务完成并收集结果
        return futures.stream()
                .map(CompletableFuture::join)
                .collect(Collectors.toList());
    }

    private JSONObject executeQuery(String sqlKey, String sqlStr) {
        JSONObject result = new JSONObject();
        try {

            long startTime = System.currentTimeMillis();
            // 执行 SQL 查询
            List<Map<String, Object>> resultList = sqlExecuteMapper.sqlQuery(sqlStr);
            long endTime = System.currentTimeMillis();
            logger.info("sql执行时长:{}ms  执行sql:{}", (endTime - startTime), sqlStr);

            // 将结果列表转换为 JSONArray
            JSONArray jsonArray = JSON.parseArray(JSON.toJSONString(resultList));
            // 执行 SQL 查询并转换为 JSON 字符串
            result.put(sqlKey, jsonArray);
        } catch (Exception ex) {
            throw new SqlException(sqlKey + ": " + sqlStr, ex);
        }
        return result;
    }
}
